@@ -185,8 +185,25 @@ class Resolver:
             filename = os.path.normpath(
                 os.path.join(self.current_directory, filename))
 
-        files = [(name, filename)]
-        if short_name:
+        if not short_name:
+            try_filename = True
+            try_short_filename = False
+        elif item.source:
+            # If the import has a source path, we can use it to eliminate
+            # filenames that don't match.
+            source_filename, _ = os.path.splitext(item.source)
+            dirname, basename = os.path.split(source_filename)
+            if basename == "__init__":
+                source_filename = dirname
+            try_filename = source_filename.endswith(filename)
+            try_short_filename = not try_filename
+        else:
+            try_filename = try_short_filename = True
+
+        files = []
+        if try_filename:
+            files.append((name, filename))
+        if try_short_filename:
             short_filename = os.path.dirname(filename)
             files.append((short_name, short_filename))
 
